# Generated by Django 4.2.16 on 2026-01-18 08:55

from django.db import migrations

from ..tasks.updates import clear_all_skill_caches


def clear_cache(apps, schema_editor):
    print(clear_all_skill_caches())


class Migration(migrations.Migration):
    dependencies = [
        ("corptools", "0127_alter_corporationaudit_options"),
    ]

    operations = [
        migrations.RunPython(clear_cache, clear_cache)
    ]
