# Generated by Django 2.2.12 on 2020-08-10 13:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0022_mapsystemgate_skilllist'),
    ]

    operations = [
        migrations.AlterField(
            model_name='mapsystemgate',
            name='from_solar_system',
            field=models.BigIntegerField(),
        ),
        migrations.AlterField(
            model_name='mapsystemgate',
            name='to_solar_system',
            field=models.BigIntegerField(),
        ),
    ]
