# Generated by Django 3.1.1 on 2020-09-29 01:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0034_auto_20200928_0005'),
    ]

    operations = [
        migrations.AlterField(
            model_name='charactermarketorder',
            name='escrow',
            field=models.DecimalField(
                decimal_places=2, default=None, max_digits=20, null=True),
        ),
        migrations.AlterField(
            model_name='corporationmarketorder',
            name='escrow',
            field=models.DecimalField(
                decimal_places=2, default=None, max_digits=20, null=True),
        ),
    ]
