# Generated by Django 4.2.16 on 2025-08-01 09:25

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("corptools", "0124_characterbountyevent_characterbountystat"),
    ]

    operations = [
        migrations.AddField(
            model_name="corporationaudit",
            name="last_change_industry_jobs",
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name="corporationaudit",
            name="last_update_industry_jobs",
            field=models.DateTimeField(blank=True, default=None, null=True),
        ),
        migrations.CreateModel(
            name="CorporationIndustryJob",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("activity_id", models.IntegerField()),
                ("blueprint_id", models.BigIntegerField()),
                ("blueprint_location_id", models.BigIntegerField()),
                ("blueprint_type_id", models.BigIntegerField()),
                (
                    "completed_character_id",
                    models.BigIntegerField(
                        blank=True, default=None, null=True),
                ),
                (
                    "completed_date",
                    models.DateTimeField(blank=True, default=None, null=True),
                ),
                (
                    "cost",
                    models.DecimalField(
                        blank=True,
                        decimal_places=2,
                        default=None,
                        max_digits=20,
                        null=True,
                    ),
                ),
                ("duration", models.IntegerField()),
                ("end_date", models.DateTimeField()),
                ("facility_id", models.BigIntegerField()),
                ("installer_id", models.IntegerField()),
                ("job_id", models.IntegerField()),
                (
                    "licensed_runs",
                    models.IntegerField(blank=True, default=None, null=True),
                ),
                ("location_id", models.BigIntegerField()),
                ("output_location_id", models.BigIntegerField()),
                (
                    "pause_date",
                    models.DateTimeField(blank=True, default=None, null=True),
                ),
                ("probability", models.FloatField(
                    blank=True, default=None, null=True)),
                ("product_type_id", models.IntegerField()),
                ("runs", models.IntegerField()),
                ("start_date", models.DateTimeField()),
                ("status", models.CharField(max_length=15)),
                (
                    "successful_runs",
                    models.IntegerField(blank=True, default=None, null=True),
                ),
                (
                    "blueprint_type_name",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="corp_blueprint_type",
                        to="corptools.eveitemtype",
                    ),
                ),
                (
                    "corporation",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="corptools.corporationaudit",
                    ),
                ),
                (
                    "product_type_name",
                    models.ForeignKey(
                        default=None,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="corp_product_type",
                        to="corptools.eveitemtype",
                    ),
                ),
            ],
        ),
    ]
