"""
This module is the core of the CorpTools Ecosystem
"""
__appname__ = "corptools"
__version__ = "3.0.0b5"
__url__ = "https://github.com/Solar-Helix-Independent-Transport/allianceauth-corp-tools"
__compat_date__ = "2025-09-30"
