# Generated by Django 4.2.13 on 2024-05-27 07:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0103_currentshipfilter'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='currentshipfilter',
            name='categories',
        ),
        migrations.AlterField(
            model_name='currentshipfilter',
            name='groups',
            field=models.ManyToManyField(blank=True, help_text='Filter on Ship Groups.', to='corptools.eveitemgroup'),
        ),
        migrations.AlterField(
            model_name='currentshipfilter',
            name='types',
            field=models.ManyToManyField(blank=True, help_text='Filter on Ship Types.', to='corptools.eveitemtype'),
        ),
    ]
