# Generated by Django 4.2.16 on 2025-02-25 12:01

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0114_alter_skilllist_skill_list'),
    ]

    operations = [
        migrations.CreateModel(
            name='AssetCoordiante',
            fields=[
                ('item', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE,
                 primary_key=True, related_name='coordinate', serialize=False, to='corptools.corpasset')),
                ('x', models.FloatField()),
                ('y', models.FloatField()),
                ('z', models.FloatField()),
            ],
        ),
    ]
