# Generated by Django 4.2.16 on 2026-02-08 08:49

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("corptools", "0128_clear_skill_cache"),
    ]

    operations = [
        migrations.AlterField(
            model_name="notification",
            name="id",
            field=models.BigAutoField(primary_key=True, serialize=False),
        ),
    ]
