# Generated by Django 4.2.23 on 2025-09-15 16:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0125_corporationaudit_last_change_industry_jobs_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='assetsfilter',
            name='reversed_logic',
            field=models.BooleanField(default=False, help_text='Negate the value of the filter, i.e. check for absence of assets'),
        ),
    ]
