# Generated by Django 3.2.8 on 2022-01-09 01:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0060_bridgeozonelevel'),
    ]

    operations = [
        migrations.CreateModel(
            name='NotificationText',
            fields=[
                ('notification_id', models.BigIntegerField(
                    primary_key=True, serialize=False)),
                ('notification_text', models.TextField(default=None, null=True)),
            ],
        ),
    ]
