# Generated by Django 3.1.1 on 2021-01-12 13:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0040_auto_20210112_0449'),
    ]

    operations = [
        migrations.CreateModel(
            name='TimeInCorpFilter',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=500)),
                ('description', models.CharField(max_length=500)),
                ('days_in_corp', models.IntegerField(default=30)),
            ],
            options={
                'verbose_name': "Smart Filter: Main's Time in Corp",
                'verbose_name_plural': "Smart Filter: Main's Time in Corp",
            },
        ),
        migrations.AlterModelOptions(
            name='assetsfilter',
            options={'verbose_name': 'Smart Filter: Assets in Locations',
                     'verbose_name_plural': 'Smart Filter: Assets in Locations'},
        ),
        migrations.AlterField(
            model_name='skillfilter',
            name='required_skill_lists',
            field=models.ManyToManyField(blank=True, to='corptools.SkillList'),
        ),
    ]
