# Generated by Django 4.2.16 on 2025-11-09 02:00

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("corptools", "0126_assetsfilter_reversed_logic"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="corporationaudit",
            options={
                "permissions": (
                    ("own_corp_manager", "Can access own corporations's data."),
                    (
                        "alliance_corp_manager",
                        "Can access other corporations's data for own alliance.",
                    ),
                    (
                        "state_corp_manager",
                        "Can access other corporations's data for own state.",
                    ),
                    ("global_corp_manager", "Can access all corporations's data."),
                    (
                        "show_if_director",
                        "Can access all corporations's where character is a director.",
                    ),
                )
            },
        ),
    ]
