# Generated by Django 3.2.5 on 2021-08-27 12:38

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0058_auto_20210808_0635'),
    ]

    operations = [
        migrations.AddField(
            model_name='invtypematerials',
            name='met_type',
            field=models.ForeignKey(default=None, null=True, on_delete=django.db.models.deletion.SET_NULL,
                                    related_name='met_type', to='corptools.eveitemtype'),
        ),
    ]
