# Generated by Django 4.2.16 on 2024-09-23 00:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0110_rename_oreprice_typeprice'),
    ]

    operations = [
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_assets',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Asset Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_clones',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Clone Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_contacts',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Contact Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_contracts',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Contracts Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_indy',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character industry Job Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_location',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Location Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_loyaltypoints',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Loyalty Point Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_mails',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Mail Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_mining',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Mining Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_notif',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Notification Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_pub_data',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Pulic History Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_roles',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Roles Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_skills',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character SKill Data'),
        ),
        migrations.AddField(
            model_name='corptoolsconfiguration',
            name='disable_update_wallet',
            field=models.BooleanField(blank=True, default=False, help_text='Temporarily disable the ESI pulls for Character Wallet Data'),
        ),
    ]
