# Generated by Django 4.0.2 on 2022-08-20 09:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0075_remove_notification_corptools_n_notific_77c7f2_idx'),
    ]

    operations = [
        migrations.RemoveIndex(
            model_name='characterasset',
            name='corptools_c_type_id_05ccc2_idx',
        ),
        migrations.RemoveIndex(
            model_name='corpasset',
            name='corptools_c_type_id_e9d79e_idx',
        ),
    ]
