# Generated by Django 4.2.16 on 2025-02-23 22:13

from django.db import migrations, models

import corptools.models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0113_timeincorpfilter_reversed_logic'),
    ]

    operations = [
        migrations.AlterField(
            model_name='skilllist',
            name='skill_list',
            field=models.TextField(default='', null=True, validators=[
                                   corptools.models.valid_skills]),
        ),
    ]
