# Generated by Django 4.2.13 on 2024-05-27 05:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0102_corporationaudit_last_change_contracts_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='CurrentShipFilter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=500)),
                ('description', models.CharField(max_length=500)),
                ('count_message_only', models.BooleanField(default=False)),
                ('categories', models.ManyToManyField(blank=True, help_text='Filter on Asset Categories.', to='corptools.eveitemcategory')),
                ('constellations', models.ManyToManyField(blank=True, help_text='Limit filter to specific constellations', to='corptools.mapconstellation')),
                ('groups', models.ManyToManyField(blank=True, help_text='Filter on Asset Groups.', to='corptools.eveitemgroup')),
                ('regions', models.ManyToManyField(blank=True, help_text='Limit filter to specific regions', to='corptools.mapregion')),
                ('systems', models.ManyToManyField(blank=True, help_text='Limit filter to specific systems', to='corptools.mapsystem')),
                ('types', models.ManyToManyField(blank=True, help_text='Filter on Asset Types.', to='corptools.eveitemtype')),
            ],
            options={
                'verbose_name': 'Smart Filter: The Current Ship being Flown',
                'verbose_name_plural': 'Smart Filter: The Current Ship being Flown',
            },
        ),
    ]
