.spinner-size{height:30px;width:30px;margin:200px}.error-size{width:100px;height:100px;margin:200px 200px 15px}.text-margin{margin-bottom:185px}@keyframes shake{10%,90%{transform:translate3d(-1px,0,0)}20%,80%{transform:translate3d(2px,0,0)}30%,50%,70%{transform:translate3d(-4px,0,0)}40%,60%{transform:translate3d(4px,0,0)}}.error-anim{animation:shake .82s cubic-bezier(.36,.07,.19,.97) both;animation-iteration-count:infinite}.padded-label{margin:5px}@keyframes spin{0%{transform:rotate(0)}to{transform:rotate(359deg)}}.glyphicon-refresh-animate{-animation:spin 2s infinite linear;-ms-animation:spin 2s infinite linear;-webkit-animation:spin 2s infinite linear;-moz-animation:spin 2s infinite linear}.menu-spinner-size{height:30px;width:30px;margin-top:12.5px}:root{--main-color: #eeeeee33;--main-color-hover: rgb(104, 117, 217, .2);--track-color: #ffffff33}.horizontal-slider{max-width:100%;margin:auto}.thumb.active{background-color:gray}.track.track{background:var(--track-color);border:1px solid #666;height:50px}.track.track-1{background:var(--track-color)!important;z-index:99999999!important;height:50px!important}.slider .track{height:4px}.slider .thumb{cursor:pointer;background:var(--main-color);width:20px;height:50px;border-radius:0%;border:2px solid #fff;outline:none}.slider .thumb-0{border-right:0}.slider .thumb-1{border-left:0}.char-portrait{width:300px;height:300px;position:relative}.corp-logo{position:absolute;bottom:0;left:0;margin:5px;width:90px;height:90px}.alli-logo{position:absolute;bottom:0;right:0;margin:5px;width:90px;height:90px}.doctrine-button{width:23%;margin:5px}.flex-doctrine-btn-name{flex:5}.flex-doctrine-btn-copy{flex:1;border:2px solid #d9534f}@media only screen and (max-width: 1900px){.doctrine-button{width:30%}}@media only screen and (min-width: 1900px){.doctrine-button{width:17%}}@media only screen and (max-width: 800px){.doctrine-button{width:100%}}.flex-five{flex:5;justify-content:space-evenly}.flex-one{flex:1}.doctrine-portrait{width:300px;height:300px}*{margin:0;padding:0}.flex{display:flex;height:80px;align-items:center}.child{margin-right:20px}.child-end{margin-left:auto}.fill-width{overflow:auto}.float-left{float:left}.flex-container{display:flex;align-items:center;justify-content:center;align-content:center;flex-wrap:wrap;flex-direction:row}.flex-wide{justify-content:space-between;flex-wrap:nowrap}.flex-wide-child{width:750px}.flex-container-pull-right{display:flex;align-items:flex-end;justify-content:center;align-content:center;flex-direction:row}.flex-container-vert-fill{display:flex;align-items:stretch;justify-content:center;align-content:stretch;flex-wrap:wrap;flex-direction:column}.flex-child{margin:10px}.flex-body{align-items:center;justify-content:center;flex-direction:column}.table-div{height:250px;width:400px;overflow-y:auto}.skill-queue{height:400px}.table-div-fill{width:auto}.table-div::-webkit-scrollbar{width:3;background:transparent}.table-div::-webkit-scrollbar-thumb{background-color:#375a7f4b;border-left:15px solid transparent;background-clip:padding-box}.no-wrap{white-space:nowrap}@media only screen and (max-width: 1400px){.info-hide{display:none}}@media only screen and (max-width: 1000px){.association-hide{display:none}}.flex-label-container{display:flex;flex-direction:row;flex-wrap:nowrap;justify-content:flex-start;align-items:stretch;align-content:stretch;flex:1;padding:5px}.flex-label{margin:2px}.flex-select{flex:1;margin:2px}.flex-group-block{align-items:flex-start;justify-content:flex-start;align-content:flex-start}.flex-skill-block{width:33%;padding-left:10px;padding-right:10px;display:flex;flex-direction:row;flex-wrap:nowrap;justify-content:space-between}@media only screen and (max-width: 1400px){.flex-skill-block{width:50%;display:flex;flex-direction:row;flex-wrap:nowrap;justify-content:space-between}}@media only screen and (max-width: 1000px){.flex-skill-block{width:100%;display:flex;flex-direction:row;flex-wrap:nowrap;justify-content:space-between}}.no-margin{margin:0!important}.flex-space-between{justify-content:space-between}.full-width{width:100%}
