# Generated by Django 4.2.16 on 2025-03-07 08:30

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0115_assetcoordiante'),
    ]

    operations = [
        migrations.CreateModel(
            name='Starbase',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('starbase_id', models.BigIntegerField()),
                ('onlined_since', models.DateTimeField(default=None, null=True)),
                ('reinforced_until', models.DateTimeField(default=None, null=True)),
                ('unanchor_at', models.DateTimeField(default=None, null=True)),
                ('allow_alliance_members', models.BooleanField(default=False)),
                ('allow_corporation_members', models.BooleanField(default=False)),
                ('attack_if_at_war', models.BooleanField(default=False)),
                ('attack_if_other_security_status_dropping',
                 models.BooleanField(default=False)),
                ('use_alliance_standings', models.BooleanField(default=False)),
                ('attack_security_status_threshold',
                 models.FloatField(default=None, null=True)),
                ('attack_standing_threshold',
                 models.FloatField(default=None, null=True)),
                ('state', models.CharField(choices=[('online', 'online'), ('offline', 'offline'), (
                    'onlining', 'onlining'), ('reinforced', 'reinforced'), ('unanchoring', 'unanchoring')], max_length=12)),
                ('anchor', models.CharField(choices=[('alliance_member', 'alliance_member'), ('config_starbase_equipment_role', 'config_starbase_equipment_role'), (
                    'corporation_member', 'corporation_member'), ('starbase_fuel_technician_role', 'starbase_fuel_technician_role')], default=None, max_length=31, null=True)),
                ('online', models.CharField(choices=[('alliance_member', 'alliance_member'), ('config_starbase_equipment_role', 'config_starbase_equipment_role'), (
                    'corporation_member', 'corporation_member'), ('starbase_fuel_technician_role', 'starbase_fuel_technician_role')], default=None, max_length=31, null=True)),
                ('offline', models.CharField(choices=[('alliance_member', 'alliance_member'), ('config_starbase_equipment_role', 'config_starbase_equipment_role'), (
                    'corporation_member', 'corporation_member'), ('starbase_fuel_technician_role', 'starbase_fuel_technician_role')], default=None, max_length=31, null=True)),
                ('unanchor', models.CharField(choices=[('alliance_member', 'alliance_member'), ('config_starbase_equipment_role', 'config_starbase_equipment_role'), (
                    'corporation_member', 'corporation_member'), ('starbase_fuel_technician_role', 'starbase_fuel_technician_role')], default=None, max_length=31, null=True)),
                ('fuel_bay_take', models.CharField(choices=[('alliance_member', 'alliance_member'), ('config_starbase_equipment_role', 'config_starbase_equipment_role'), (
                    'corporation_member', 'corporation_member'), ('starbase_fuel_technician_role', 'starbase_fuel_technician_role')], default=None, max_length=31, null=True)),
                ('fuel_bay_view', models.CharField(choices=[('alliance_member', 'alliance_member'), ('config_starbase_equipment_role', 'config_starbase_equipment_role'), (
                    'corporation_member', 'corporation_member'), ('starbase_fuel_technician_role', 'starbase_fuel_technician_role')], default=None, max_length=31, null=True)),
                ('fuels', models.TextField(blank=True, default='', null=True)),
                ('corporation', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.corporationaudit')),
                ('moon', models.ForeignKey(default=None, null=True,
                 on_delete=django.db.models.deletion.CASCADE, to='corptools.mapsystemmoon')),
                ('system', models.ForeignKey(default=None, null=True,
                 on_delete=django.db.models.deletion.CASCADE, to='corptools.mapsystem')),
                ('type_name', models.ForeignKey(default=None, null=True,
                 on_delete=django.db.models.deletion.CASCADE, to='corptools.eveitemtype')),
            ],
        ),
    ]
