# Generated by Django 4.0.10 on 2023-05-22 10:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0087_characteraudit_last_update_indy'),
    ]

    operations = [
        migrations.AlterField(
            model_name='characterindustryjob',
            name='end_date',
            field=models.DateTimeField(),
        ),
    ]
