# Generated by Django 4.2.16 on 2025-06-07 00:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0121_charassetcoordiante'),
    ]

    operations = [
        migrations.AddField(
            model_name='characteraudit',
            name='total_logins',
            field=models.IntegerField(blank=True, default=None, null=True),
        ),
    ]
