# Generated by Django 4.2.16 on 2025-06-13 09:15

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0123_characterwalletjournalentry_processed_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='CharacterBountyEvent',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('message', models.TextField()),
                ('entry', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE,
                 related_name='msg', to='corptools.characterwalletjournalentry')),
            ],
        ),
        migrations.CreateModel(
            name='CharacterBountyStat',
            fields=[
                ('id', models.AutoField(auto_created=True,
                 primary_key=True, serialize=False, verbose_name='ID')),
                ('qty', models.IntegerField()),
                ('event', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                 related_name='stats', to='corptools.characterbountyevent')),
                ('type_name', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE, to='corptools.eveitemtype')),
            ],
        ),
    ]
