# Generated by Django 4.2.16 on 2025-06-09 10:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0122_characteraudit_total_logins'),
    ]

    operations = [
        migrations.AddField(
            model_name='characterwalletjournalentry',
            name='processed',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='corporationwalletjournalentry',
            name='processed',
            field=models.BooleanField(default=False),
        ),
    ]
