# Generated by Django 3.2.8 on 2022-01-09 11:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0067_remove_notification_notification_text'),
    ]

    operations = [
        migrations.RenameField(
            model_name='notification',
            old_name='note_text',
            new_name='notification_text',
        ),
    ]
