# Generated by Django 4.2.16 on 2025-02-11 11:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0112_alter_corptoolsconfiguration_holding_corps'),
    ]

    operations = [
        migrations.AddField(
            model_name='timeincorpfilter',
            name='reversed_logic',
            field=models.BooleanField(
                default=False, help_text='If set all members less than the days in corp will pass the test.'),
        ),
    ]
