# Generated by Django 2.2.12 on 2020-08-10 13:33

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0021_auto_20200804_0941'),
    ]

    operations = [
        migrations.CreateModel(
            name='SkillList',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('last_update', models.DateTimeField(auto_now=True)),
                ('Name', models.CharField(default=None, max_length=500, null=True)),
                ('skill_list', models.TextField(default='', null=True)),
                ('eft', models.TextField(default='', null=True)),
            ],
        ),
        migrations.CreateModel(
            name='MapSystemGate',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True, serialize=False, verbose_name='ID')),
                ('from_solar_system', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                                        related_name='from_system', to='corptools.MapSystem')),
                ('to_solar_system', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                                      related_name='to_system', to='corptools.MapSystem')),
            ],
        ),
    ]
